# coding: utf-8

"""
    Aron API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 1.0.0
    Generated by: https://openapi-generator.tech
"""


try:
    from inspect import getfullargspec
except ImportError:
    from inspect import getargspec as getfullargspec
import pprint
import re  # noqa: F401
import six

from openapi_client.configuration import Configuration


class OrmModelServiceGatewayTrafficSplitEntry(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'port': 'int',
        'revision_number': 'int',
        'traffic_weight': 'int'
    }

    attribute_map = {
        'port': 'port',
        'revision_number': 'revision_number',
        'traffic_weight': 'traffic_weight'
    }

    def __init__(self, port=None, revision_number=None, traffic_weight=None, local_vars_configuration=None):  # noqa: E501
        """OrmModelServiceGatewayTrafficSplitEntry - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration.get_default_copy()
        self.local_vars_configuration = local_vars_configuration

        self._port = None
        self._revision_number = None
        self._traffic_weight = None
        self.discriminator = None

        if port is not None:
            self.port = port
        if revision_number is not None:
            self.revision_number = revision_number
        if traffic_weight is not None:
            self.traffic_weight = traffic_weight

    @property
    def port(self):
        """Gets the port of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501


        :return: The port of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :rtype: int
        """
        return self._port

    @port.setter
    def port(self, port):
        """Sets the port of this OrmModelServiceGatewayTrafficSplitEntry.


        :param port: The port of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :type port: int
        """
        if (self.local_vars_configuration.client_side_validation and
                port is not None and port > 65535):  # noqa: E501
            raise ValueError("Invalid value for `port`, must be a value less than or equal to `65535`")  # noqa: E501
        if (self.local_vars_configuration.client_side_validation and
                port is not None and port < 1):  # noqa: E501
            raise ValueError("Invalid value for `port`, must be a value greater than or equal to `1`")  # noqa: E501

        self._port = port

    @property
    def revision_number(self):
        """Gets the revision_number of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501


        :return: The revision_number of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :rtype: int
        """
        return self._revision_number

    @revision_number.setter
    def revision_number(self, revision_number):
        """Sets the revision_number of this OrmModelServiceGatewayTrafficSplitEntry.


        :param revision_number: The revision_number of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :type revision_number: int
        """

        self._revision_number = revision_number

    @property
    def traffic_weight(self):
        """Gets the traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501


        :return: The traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :rtype: int
        """
        return self._traffic_weight

    @traffic_weight.setter
    def traffic_weight(self, traffic_weight):
        """Sets the traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.


        :param traffic_weight: The traffic_weight of this OrmModelServiceGatewayTrafficSplitEntry.  # noqa: E501
        :type traffic_weight: int
        """

        self._traffic_weight = traffic_weight

    def to_dict(self, serialize=False):
        """Returns the model properties as a dict"""
        result = {}

        def convert(x):
            if hasattr(x, "to_dict"):
                args = getfullargspec(x.to_dict).args
                if len(args) == 1:
                    return x.to_dict()
                else:
                    return x.to_dict(serialize)
            else:
                return x

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            attr = self.attribute_map.get(attr, attr) if serialize else attr
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: convert(x),
                    value
                ))
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], convert(item[1])),
                    value.items()
                ))
            else:
                result[attr] = convert(value)

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrmModelServiceGatewayTrafficSplitEntry):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrmModelServiceGatewayTrafficSplitEntry):
            return True

        return self.to_dict() != other.to_dict()
